#!/usr/bin/env python2

import urllib
import hashlib
import sys
import os
from optparse import OptionParser


class AppURLopener(urllib.FancyURLopener):
    version = "Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US; rv:1.8.0.7) Gecko/20060909 Firefox/1.5.0.7"

    def prompt_user_passwd(self, host, realm):
        raise Exception()


urllib._urlopener = AppURLopener()


def file_md5(file):
    m = hashlib.md5()
    m.update(open(file).read())
    return m.hexdigest()


def main():
    parser = OptionParser(
        usage="usage: %prog URI dest [md5sum]",
        prog=sys.argv[0])
    options, args = parser.parse_args()
    md5sum = None

    if len(args) == 2:
        uri, dest = args
    elif len(args) == 3:
        uri, dest, md5 = args
    else:
        parser.error("Wrong arguments")

    fresh = False
    if not os.path.exists(dest):
        print("Downloading from %s to %s..." % (uri, dest),)
        sys.stdout.flush()
        urllib.urlretrieve(uri, dest)
        print("done")
        fresh = True

    if md5sum:
        print("Computing md5sum on downloaded file",)
        sys.stdout.flush()
        checksum = file_md5(dest)
        print("done")

        if checksum != md5sum:
            if not fresh:
                print("Checksum mismatch (%s != %s), re-downloading file %s" % (checksum, md5sum, dest),)
                sys.stdout.flush()
                os.remove(dest)
                urllib.urlretrieve(uri, dest)
                print("done")

                print("Computing md5sum on downloaded file",)
                sys.stdout.flush()
                checksum = file_md5(dest)
                print("done")

                if checksum != md5sum:
                    print("ERROR, checksum mismatch (%s != %s) on %d", (checksum, md5sum, dest))
                    return 1
    return 0


if __name__ == '__main__':
    try:
        sys.exit(main())
    except Exception as e:
        print("ERROR, ", e)
        sys.exit(1)



#!/usr/bin/env python2


import sys
import unittest

if __name__ == "__main__":
    if len(sys.argv) == 1:
        print("Usage: %s file" % sys.argv[0])
        sys.exit(1)

    test_file = sys.argv[1]
    sys.argv = sys.argv[1:]
    execfile(test_file)



#!/usr/bin/env python
from os.path import dirname, join, abspath
import sqlite3
import sys
import os
import re


def is_cibuildwheel():
    """Check if run with cibuildwheel."""
    return 'CIBUILDWHEEL' in os.environ


def temp_rename_kernprof(repo_dir):
    """
    Hacky workaround so kernprof.py doesn't get covered twice (installed and local).
    This needed to combine the .coverage files, since file paths need to be unique.

    """
    original_path = repo_dir + '/kernprof.py'
    tmp_path = original_path + '.tmp'
    if os.path.isfile(original_path):
        os.rename(original_path, tmp_path)
    elif os.path.isfile(tmp_path):
        os.rename(tmp_path, original_path)


def replace_docker_path(path, runner_project_dir):
    """Update path to a file installed in a temp venv to runner_project_dir."""
    pattern = re.compile(r"\/tmp\/.+?\/site-packages")
    return pattern.sub(runner_project_dir, path)


def update_coverage_file(coverage_path, runner_project_dir):
    """
    Since the paths inside of docker vary from the runner paths,
    the paths in the .coverage file need to be adjusted to combine them,
    since 'coverage combine <folder>' checks if the file paths exist.
    """
    try:
        sqliteConnection = sqlite3.connect(coverage_path)
        cursor = sqliteConnection.cursor()
        print('Connected to Coverage SQLite')

        read_file_query = 'SELECT id, path from file'
        cursor.execute(read_file_query)

        old_records = cursor.fetchall()
        new_records = [(replace_docker_path(path, runner_project_dir), _id) for _id, path in old_records]
        print('Updated coverage file paths:\n', new_records)

        sql_update_query = 'Update file set path = ? where id = ?'
        cursor.executemany(sql_update_query, new_records)
        sqliteConnection.commit()
        print('Coverage Updated successfully')
        cursor.close()

    except sqlite3.Error as error:
        print('Failed to coverage: ', error)
    finally:
        if sqliteConnection:
            sqliteConnection.close()
            print('The sqlite connection is closed')


def copy_coverage_cibuildwheel_docker(runner_project_dir):
    """
    When run with cibuildwheel under linux, the tests run in the folder /project
    inside docker and the coverage files need to be copied to the output folder.
    """
    coverage_path = '/project/tests/.coverage'
    if os.path.isfile(coverage_path):
        update_coverage_file(coverage_path, runner_project_dir)
        env_hash = hash((sys.version, os.environ.get('AUDITWHEEL_PLAT', '')))
        os.makedirs('/output', exist_ok=True)
        os.rename(coverage_path, '/output/.coverage.{}'.format(env_hash))


if __name__ == '__main__':
    cwd = os.getcwd()
    repo_dir = abspath(dirname(__file__))
    test_dir = join(repo_dir, 'tests')
    print('cwd = {!r}'.format(cwd))

    import pytest

    # Prefer testing the installed version, but fallback to testing the
    # development version.
    try:
        import ubelt as ub
    except ImportError:
        print('running this test script requires ubelt')
        raise
    package_name = 'pyflann_ibeis'
    # Statically check if ``package_name`` is installed outside of the repo.
    # To do this, we make a copy of PYTHONPATH, remove the repodir, and use
    # ubelt to check to see if ``package_name`` can be resolved to a path.
    temp_path = list(map(abspath, sys.path))
    if repo_dir in temp_path:
        temp_path.remove(repo_dir)
    modpath = ub.modname_to_modpath(package_name, sys_path=temp_path)
    if modpath is not None:
        # If it does, then import it. This should cause the installed version
        # to be used on further imports even if the repo_dir is in the path.
        print(f'Using installed version of {package_name}')
        module = ub.import_module_from_path(modpath, index=0)
        print('Installed module = {!r}'.format(module))
    else:
        print(f'No installed version of {package_name} found')

    try:
        print('Changing dirs to test_dir={!r}'.format(test_dir))
        os.chdir(test_dir)

        pytest_args = [
            '--cov-config', '../pyproject.toml',
            '--cov-report', 'html',
            '--cov-report', 'term',
            '--cov-report', 'xml',
            '--cov=' + package_name,
        ]
        if is_cibuildwheel():
            pytest_args.append('--cov-append')

        pytest_args = pytest_args + sys.argv[1:]
        sys.exit(pytest.main(pytest_args))
    finally:
        os.chdir(cwd)
        if is_cibuildwheel():
            # for CIBW under linux
            copy_coverage_cibuildwheel_docker(f'/home/runner/work/{package_name}/{package_name}')
        print('Restoring cwd = {!r}'.format(cwd))



#!/usr/bin/env python
from pyflann_ibeis import FLANN, index_type
from numpy import arange
from numpy.random import rand, randn
import unittest


class Test_PyFLANN_nn(unittest.TestCase):

    def setUp(self):
        self.nn = FLANN()


class Test_PyFLANN_nn_index(unittest.TestCase):

    def testnn_index_save_kdtree_1(self):
        self.run_nn_index_save_perturbed(64, 1000, algorithm="kdtree", trees=1)

    def testnn_index_save_kdtree_4(self):
        self.run_nn_index_save_perturbed(64, 1000, algorithm="kdtree", trees=4)

    def testnn_index_save_kdtree_10(self):
        self.run_nn_index_save_perturbed(
            64, 1000, algorithm="kdtree", trees=10)

    def testnn_index_save_kmeans_2(self):
        self.run_nn_index_save_perturbed(
            64, 1000, algorithm="kmeans", branching=2, iterations=11)

    def testnn_index_save_kmeans_16(self):
        self.run_nn_index_save_perturbed(
            64, 1000, algorithm="kmeans", branching=16, iterations=11)

    def testnn_index_save_kmeans_32(self):
        self.run_nn_index_save_perturbed(
            64, 1000, algorithm="kmeans", branching=32, iterations=11)

    def testnn_index_save_kmeans_64(self):
        self.run_nn_index_save_perturbed(
            64, 1000, algorithm="kmeans", branching=64, iterations=11)

    def testnn__save_kdtree_1(self):
        self.run_nn_index_save_rand(
            64,
            10000,
            1000,
            algorithm="kdtree",
            trees=1,
            checks=128)

    def testnn__save_kdtree_4(self):
        self.run_nn_index_save_rand(
            64,
            10000,
            1000,
            algorithm="kdtree",
            trees=4,
            checks=128)

    def testnn__save_kdtree_10(self):
        self.run_nn_index_save_rand(
            64,
            10000,
            1000,
            algorithm="kdtree",
            trees=10,
            checks=128)

    def testnn__save_kmeans_2(self):
        self.run_nn_index_save_rand(
            64,
            1000,
            1000,
            algorithm="kmeans",
            branching=2,
            iterations=11,
            checks=64)

    def testnn__save_kmeans_8(self):
        self.run_nn_index_save_rand(
            64,
            10000,
            1000,
            algorithm="kmeans",
            branching=8,
            iterations=11,
            checks=32)

    def testnn__save_kmeans_16(self):
        self.run_nn_index_save_rand(
            64,
            10000,
            1000,
            algorithm="kmeans",
            branching=16,
            iterations=11,
            checks=40)

    def testnn__save_kmeans_32(self):
        self.run_nn_index_save_rand(
            64,
            10000,
            1000,
            algorithm="kmeans",
            branching=32,
            iterations=11,
            checks=56)

    def run_nn_index_save_perturbed(self, dim, N, **kwargs):

        x = rand(N, dim)

        nn = FLANN()
        nn.build_index(x, **kwargs)
        nn.save_index("index.dat")
        nn.delete_index()

        nn = FLANN()
        nn.load_index("index.dat", x)
        x_query = x + randn(x.shape[0], x.shape[1]) * 0.0001 / dim
        nnidx, nndist = nn.nn_index(x_query)
        correct = all(nnidx == arange(N, dtype=index_type))

        nn.delete_index()
        self.assertTrue(correct)

    def run_nn_index_save_rand(self, dim, N, Nq, **kwargs):

        x = rand(N, dim)
        x_query = rand(Nq, dim)

        # build index, search and delete it
        nn = FLANN()
        nn.build_index(x, **kwargs)
        nnidx, nndist = nn.nn_index(x_query, checks=kwargs["checks"])
        nn.save_index("index.dat")
        del nn

        # now reload index and search again
        nn = FLANN()
        nn.load_index("index.dat", x)
        nnidx2, nndist2 = nn.nn_index(x_query, checks=kwargs["checks"])
        del nn

        correct = all(nnidx == nnidx2)
        self.assertTrue(correct)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
import numpy as np
from pyflann_ibeis import FLANN
import unittest
import time


class Test_PyFLANN_clustering(unittest.TestCase):

    def setUp(self):
        self.nn = FLANN(iterations=11)

    ##########################################################################

    def test_Rand(self):
        x = np.random.rand(100, 10000)
        nK = 10
        centroids = self.nn.kmeans(x, nK)
        self.assertTrue(len(centroids) == nK)

    def test2d_small(self):
        self.__nd_random_clustering_test(2, 2)

    def test3d_small(self):
        self.__nd_random_clustering_test(3, 3)

    def test4d_small(self):
        self.__nd_random_clustering_test(4, 4)

    def test3d_large(self):
        self.__nd_random_clustering_test(3, 3, 1000)

    def test10d_large(self):
        self.__nd_random_clustering_test(10, 2, 10)

    def test500d(self):
        self.__nd_random_clustering_test(500, 2, 10)

    def __nd_random_clustering_test(self, dim, N, dup=1):
        """
        Make a set of random points, then pass the same ones to the
        query points.  Each point should be closest to itself.
        """
        np.random.seed(0)
        x = np.random.rand(N, dim)
        xc = np.concatenate(tuple([x for i in range(dup)]))

        if dup > 1:
            xc += np.random.randn(xc.shape[0], xc.shape[1]) * 0.000001 / dim

        # rnseed = int(time.time())
        centroids = self.nn.kmeans(
            xc[np.random.permutation(len(xc))], N, centers_init="random", random_seed=2)
        mindists = np.array([[sum((d1 - d2)**2) for d1 in x]
                             for d2 in centroids]).min(0)
        # print mindists
        # FIXME: this can randomly fail, loosen the bounds
        for m in mindists:
            self.assertAlmostEqual(m, 0.0, 1)

        # rnseed = int(time.time())
        centroids = self.nn.kmeans(
            xc[np.random.permutation(len(xc))], N, centers_init="gonzales", random_seed=2)
        mindists = np.array([[sum((d1 - d2)**2) for d1 in x]
                             for d2 in centroids]).min(0)
        # print mindists
        for m in mindists:
            self.assertAlmostEqual(m, 0.0, 1)

        centroids = self.nn.kmeans(
            xc[np.random.permutation(len(xc))], N, centers_init="kmeanspp", random_seed=2)
        mindists = np.array([[sum((d1 - d2)**2) for d1 in x]
                             for d2 in centroids]).min(0)
        # print mindists
        for m in mindists:
            print('warning? m = {!r}'.format(m))
            # self.assertAlmostEqual(m, 0.0, 1)

    def testrandomnumber_same(self):
        """
        self = Test_PyFLANN_clustering()
        self.setUp()
        """
        import pytest
        pytest.skip('broken, but not worth fixing')

        data = np.random.rand(1000, 2)  # Random, so we can get a lot of local minima

        rnseed = int(time.time())
        cl1 = self.nn.kmeans(data, 50, random_seed=rnseed)
        cl2 = self.nn.kmeans(data, 50, random_seed=rnseed)

        self.assertTrue(np.all(cl1 == cl2))

    def testrandnumber_different(self):

        data = np.random.rand(1000, 100)  # Random, so we can get a lot of local minima

        rnseed = int(time.time())
        cl1 = self.nn.kmeans(data, 50, random_seed=rnseed)
        cl2 = self.nn.kmeans(data, 50)

        self.assertTrue(np.any(cl1 != cl2))


if __name__ == '__main__':
    """
    pytest ~/code/flann/tests/test_clustering.py --verbose
    """
    unittest.main()



def parse_authors():
    """
    Parse the git authors of a repo

    Returns:
        List[str]: list of authors

    CommandLine:
        python bin/parse_authors.py
    """
    import subprocess
    try:
        output = subprocess.check_output(['git', 'shortlog', '-s'],
                                         universal_newlines=True)
    except Exception as ex:
        print('ex = {!r}'.format(ex))
        return []
    else:
        striped_lines = (l.strip() for l in output.split('\n'))
        freq_authors = [line.split(None, 1) for line in striped_lines if line]
        freq_authors = sorted((int(f), a) for f, a in freq_authors)[::-1]
        # keep authors with uppercase letters
        authors = [a for f, a in freq_authors if a.lower() != a]
        return authors


if __name__ == '__main__':
    print(parse_authors())



#!/usr/bin/env python
import pyflann_ibeis
import numpy as np
from numpy import ones
from numpy.random import rand
import pytest
import unittest


class Test_PyFLANN_nn(unittest.TestCase):

    def setUp(self):
        self.nn = pyflann_ibeis.FLANN()


class Test_PyFLANN_nn_index(unittest.TestCase):

    def testnn_index(self):

        dim = 10
        N = 100

        x = rand(N, dim)
        nn = pyflann_ibeis.FLANN()
        nn.build_index(x)

        nnidx, nndist = nn.nn_index(x)
        correct = all(nnidx == np.arange(N, dtype=pyflann_ibeis.index_type))

        nn.delete_index()
        self.assertTrue(correct)

    def testnn_index_random_permute(self):

        numtests = 500
        dim = 10
        N = 100

        nns = [None] * numtests
        x = [rand(N, dim) for i in range(numtests)]
        correct = ones(numtests, dtype=np.bool_)

        for i in np.random.permutation(numtests):
            nns[i] = pyflann_ibeis.FLANN()
            nns[i].build_index(x[i])

            # For kicks
            if rand() < 0.5:
                nns[i].kmeans(x[i], 5)
            if rand() < 0.5:
                nns[i].nn(x[i], x[i])

        for i in np.random.permutation(numtests):
            nnidx, nndist = nns[i].nn_index(x[i])
            correct[i] = all(nnidx == np.arange(N, dtype=pyflann_ibeis.index_type))

        for i in reversed(range(numtests)):
            if rand() < 0.5:
                nns[i].delete_index()
            else:
                del nns[i]

        self.assertTrue(all(correct))

    @pytest.mark.skip('not debugging')
    def testnn_index_bad_index_call_noindex(self):
        nn = pyflann_ibeis.FLANN()
        # self.assertRaises(FLANNException, lambda: nn.nn_index(rand(5, 5)))
        import pytest
        with pytest.raises(pyflann_ibeis.FLANNException):
            nn.nn_index(rand(5, 5))

    @pytest.mark.skip('not debugging')
    def testnn_index_bad_index_call_delindex(self):
        nn = pyflann_ibeis.FLANN()
        nn.build_index(rand(5, 5))
        nn.delete_index()

        with pytest.raises(pyflann_ibeis.FLANNException):
            nn.nn_index(rand(5, 5))
        # self.assertRaises(FLANNException, lambda: nn.nn_index(rand(5, 5)))


if __name__ == '__main__':
    """
    pytest ~/code/flann/tests/test_nn_index.py
    xdoctest ~/code/flann/tests/test_nn_index.py zero
    """
    unittest.main()



#Copyright 2008-2009  Marius Muja (mariusm@cs.ubc.ca). All rights reserved.
#Copyright 2008-2009  David G. Lowe (lowe@cs.ubc.ca). All rights reserved.
#
#THE BSD LICENSE
#
#Redistribution and use in source and binary forms, with or without
#modification, are permitted provided that the following conditions
#are met:
#
#1. Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#2. Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
#IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
#OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
#NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
#THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


# This module defines exceptions that are used by the flann python bindings

class FLANNException(Exception):  # nocover
    pass
    # def __init__(self, *args):
    #     Exception.__init__(self, *args)



#!/usr/bin/env python
from guppy import hpy
from numpy.random import rand
from pyflann_ibeis import FLANN

import os
import gc

_proc_status = '/proc/%d/status' % os.getpid()

_scale = {'kB': 1024.0, 'mB': 1024.0 * 1024.0,
          'KB': 1024.0, 'MB': 1024.0 * 1024.0}


def _VmB(VmKey):
    '''Private.
    '''
    global _proc_status, _scale
    # get pseudo file  /proc/<pid>/status
    try:
        t = open(_proc_status)
        v = t.read()
        t.close()
    except BaseException:
        return 0.0  # non-Linux?
    # get VmKey line e.g. 'VmRSS:  9999  kB\n ...'
    i = v.index(VmKey)
    v = v[i:].split(None, 3)  # whitespace
    if len(v) < 3:
        return 0.0  # invalid format?
    # convert Vm value to bytes
    return float(v[1]) * _scale[v[2]]


def memory(since=0.0):
    '''Return memory usage in bytes.
    '''
    return _VmB('VmSize:') - since


def resident(since=0.0):
    '''Return resident memory usage in bytes.
    '''
    return _VmB('VmRSS:') - since


def stacksize(since=0.0):
    '''Return stack size in bytes.
    '''
    return _VmB('VmStk:') - since


if __name__ == '__main__':

    print('Profiling Memory usage for pyflann; CTRL-C to stop.')
    print('Increasing total process memory, relative to the python memory, ')
    print('implies a memory leak in the external libs.')
    print('Increasing python memory implies a memory leak in the python code.')

    h = hpy()

    while True:
        s = str(h.heap())

        print('Python: %s;    Process Total: %s' %
              (s[:s.find('\n')], memory()))

        X1 = rand(50000, 2)
        X2 = rand(50000, 2)
        pf = FLANN()
        nnlist = pf.nn(X1, X2)
        del X1
        del X2
        del nnlist
        del pf
        gc.collect()



import numpy
from numpy import ndarray
from typing import Tuple
from os import PathLike
from typing import Union
from typing import List
from typing import Any

index_type: Any


def set_distance_type(distance_type, order: int = ...) -> None:
    ...


def to_bytes(string):
    ...


class FLANN:

    def __init__(self, **kwargs) -> None:
        ...

    def __del__(self) -> None:
        ...

    def nn(self,
           pts: numpy.ndarray,
           qpts: numpy.ndarray,
           num_neighbors: int = 1,
           **kwargs) -> Tuple[ndarray, ndarray]:
        ...

    def build_index(self, pts: ndarray, **kwargs):
        ...

    def save_index(self, filename: PathLike) -> None:
        ...

    def load_index(self, filename: PathLike, pts: ndarray) -> None:
        ...

    def used_memory(self) -> int:
        ...

    def add_points(self, pts: ndarray, rebuild_threshold: float = 2.0) -> None:
        ...

    def remove_point(self, idx: int) -> None:
        ...

    def nn_index(self, qpts: ndarray, num_neighbors: int = 1, **kwargs):
        ...

    def nn_radius(self, query: ndarray, radius: float, **kwargs):
        ...

    def delete_index(self, **kwargs) -> None:
        ...

    def kmeans(self,
               pts: numpy.ndarray,
               num_clusters: int,
               max_iterations: Union[int, None] = None,
               dtype: Union[type, None] = None,
               **kwargs):
        ...

    def hierarchical_kmeans(self,
                            pts: numpy.ndarray,
                            branch_size: int,
                            num_branches: int,
                            max_iterations: Union[int, None] = None,
                            dtype: Union[type, None] = None,
                            **kwargs):
        ...

    @property
    def shape(self) -> Tuple:
        ...

    @property
    def __len__(self) -> int:
        ...

    def get_indexed_shape(self) -> Tuple[int, int]:
        ...

    def get_indexed_data(self) -> Tuple[ndarray, ndarray]:
        ...

    def used_memory_dataset(self) -> int:
        ...

    def remove_points(self, idxs: List[int]) -> None:
        ...



#!/usr/bin/env python
from pyflann_ibeis import FLANN
import unittest
import numpy as np


class Test_PyFLANN_nn(unittest.TestCase):

    def setUp(self):
        self.nn = FLANN()

    ##########################################################################
    # The typical

    def test_nn_2d_10pt_kmeans(self):
        self.__nd_random_test(2, 2, algorithm='kdtree')

    def test_nn_2d_1000pt_kmeans(self):
        self.__nd_random_test(2, 1000, algorithm='kmeans')

    def test_nn_100d_1000pt_kmeans(self):
        self.__nd_random_test(100, 1000, algorithm='kmeans')

    def test_nn_500d_100pt_kmeans(self):
        self.__nd_random_test(500, 100, algorithm='kmeans')

    def test_nn_2d_1000pt_kdtree(self):
        self.__nd_random_test(2, 1000, algorithm='kdtree')

    def test_nn_100d_1000pt_kdtree(self):
        self.__nd_random_test(100, 1000, algorithm='kdtree')

    def test_nn_500d_100pt_kdtree(self):
        self.__nd_random_test(500, 100, algorithm='kdtree')

    def test_nn_2d_1000pt_linear(self):
        self.__nd_random_test(2, 1000, algorithm='linear')

    def test_nn_100d_50pt_linear(self):
        self.__nd_random_test(100, 50, algorithm='linear')

    def test_nn_2d_1000pt_composite(self):
        self.__nd_random_test(2, 1000, algorithm='composite')

    def test_nn_100d_1000pt_composite(self):
        self.__nd_random_test(100, 1000, algorithm='composite')

    def test_nn_500d_100pt_composite(self):
        self.__nd_random_test(500, 100, algorithm='composite')

    def test_nn_multtrees_2d_1000pt_kmeans(self):
        self.__nd_random_test(2, 1000, algorithm='kmeans', trees=8)

    def test_nn_multtrees_100d_1000pt_kmeans(self):
        self.__nd_random_test(100, 1000, algorithm='kmeans', trees=8)

    def test_nn_multtrees_500d_100pt_kmeans(self):
        self.__nd_random_test(500, 100, algorithm='kmeans', trees=8)

    ##########################################################################
    # Stress it should handle

    def test_nn_stress_1d_1pt_kmeans(self):
        self.__nd_random_test(1, 1, algorithm='kmeans')

    def test_nn_stress_1d_1pt_linear(self):
        self.__nd_random_test(1, 1, algorithm='linear')

    def test_nn_stress_1d_1pt_kdtree(self):
        self.__nd_random_test(1, 1, algorithm='kdtree')

    def test_nn_stress_1d_1pt_composite(self):
        self.__nd_random_test(1, 1, algorithm='composite')

    def __nd_random_test(self, dim, N, type=np.float32,
                         num_neighbors=10, **kwargs):
        """
        Make a set of random points, then pass the same ones to the
        query points.  Each point should be closest to itself.
        """
        np.random.seed(0)
        x = np.array(np.random.rand(N, dim), dtype=type)
        perm = np.random.permutation(N)

        idx, dists = self.nn.nn(x, x[perm], **kwargs)
        self.assertTrue(all(idx == perm))

        # Make sure it's okay if we do make all the points equal
        x_mult_nn = np.concatenate([x for i in range(num_neighbors)])
        nidx, ndists = self.nn.nn(
            x_mult_nn, x, num_neighbors=num_neighbors, **kwargs)

        correctness = 0.0

        for i in range(N):
            correctness += float(len(set(nidx[i]).intersection(
                [i + n * N for n in range(num_neighbors)]))) / num_neighbors

        self.assertTrue(correctness / N >= 0.99,
                        'failed #1: N=%d,correctness=%f' % (N, correctness / N))

        # now what happens if they are slightly off
        x_mult_nn += np.random.randn(x_mult_nn.shape[0],
                                     x_mult_nn.shape[1]) * 0.0001 / dim
        n2idx, n2dists = self.nn.nn(
            x_mult_nn, x, num_neighbors=num_neighbors, **kwargs)

        for i in range(N):
            correctness += float(len(set(n2idx[i]).intersection(
                [i + n * N for n in range(num_neighbors)]))) / num_neighbors

        self.assertTrue(correctness / N >= 0.99,
                        'failed #2: N=%d,correctness=%f' % (N, correctness / N))


if __name__ == '__main__':
    """
    pytest  ~/code/flann/test/test_nn.py --verbose
    """
    unittest.main()





def main():  # nocover
    import pyflann_ibeis
    print('Looks like the imports worked')
    print('pyflann_ibeis = {!r}'.format(pyflann_ibeis))
    print('pyflann_ibeis.__file__ = {!r}'.format(pyflann_ibeis.__file__))
    print('pyflann_ibeis.__version__ = {!r}'.format(pyflann_ibeis.__version__))
    print('pyflann_ibeis.flann_ctypes.flannlib = {!r}'.format(pyflann_ibeis.flann_ctypes.flannlib))
    print('pyflann_ibeis.flann_ctypes.libpath = {!r}'.format(pyflann_ibeis.flann_ctypes.libpath))


if __name__ == '__main__':
    """
    CommandLine:
       python -m pyflann_ibeis
    """
    main()



#!/usr/bin/env python
r"""


Build Wheels:
    pip install cibuildwheel

    CIBW_BUILD_VERBOSITY=1 \
    CIBW_TEST_REQUIRES="-r requirements/tests.txt" \
    CIBW_TEST_COMMAND='python {project}/run_tests.py' \
    CIBW_SKIP='pp*' \
        cibuildwheel --config-file pyproject.toml --platform linux --arch x86_64


"""
from __future__ import absolute_import, division, print_function
import sys
from os.path import exists
from collections import OrderedDict


def native_mb_python_tag(plat_impl=None, version_info=None):
    """
    Example:
        >>> print(native_mb_python_tag())
        >>> print(native_mb_python_tag('PyPy', (2, 7)))
        >>> print(native_mb_python_tag('CPython', (3, 8)))
    """
    if plat_impl is None:
        import platform
        plat_impl = platform.python_implementation()

    if version_info is None:
        import sys
        version_info = sys.version_info

    major, minor = version_info[0:2]
    ver = '{}{}'.format(major, minor)

    if plat_impl == 'CPython':
        # TODO: get if cp27m or cp27mu
        impl = 'cp'
        if ver == '27':
            IS_27_BUILT_WITH_UNICODE = True  # how to determine this?
            if IS_27_BUILT_WITH_UNICODE:
                abi = 'mu'
            else:
                abi = 'm'
        else:
            if ver == '38':
                # no abi in 38?
                abi = ''
            else:
                abi = 'm'
        mb_tag = '{impl}{ver}-{impl}{ver}{abi}'.format(**locals())
    elif plat_impl == 'PyPy':
        abi = ''
        impl = 'pypy'
        ver = '{}{}'.format(major, minor)
        mb_tag = '{impl}-{ver}'.format(**locals())
    else:
        raise NotImplementedError(plat_impl)
    return mb_tag


def parse_version(fpath='pyflann_ibeis/__init__.py'):
    """
    Statically parse the version number from a python file


    """
    import ast
    if not exists(fpath):
        raise ValueError('fpath={!r} does not exist'.format(fpath))
    with open(fpath, 'r') as file_:
        sourcecode = file_.read()
    pt = ast.parse(sourcecode)
    class VersionVisitor(ast.NodeVisitor):
        def visit_Assign(self, node):
            for target in node.targets:
                if getattr(target, 'id', None) == '__version__':
                    self.version = node.value.s
    visitor = VersionVisitor()
    visitor.visit(pt)
    return visitor.version


def parse_long_description(fpath='README.rst'):
    """
    Reads README text, but doesn't break if README does not exist.
    """
    if exists(fpath):
        with open(fpath, 'r') as file:
            return file.read()
    return ''


def parse_requirements(fname='requirements.txt'):
    """
    Parse the package dependencies listed in a requirements file but
    strips specific versioning information.

    CommandLine:
        python -c "import setup; print(setup.parse_requirements())"
    """
    import re
    require_fpath = fname

    def parse_line(line):
        """
        Parse information from a line in a requirements text file
        """
        if line.startswith('-r '):
            # Allow specifying requirements in other files
            target = line.split(' ')[1]
            for info in parse_require_file(target):
                yield info
        elif line.startswith('-e '):
            info = {}
            info['package'] = line.split('#egg=')[1]
            yield info
        else:
            # Remove versioning from the package
            pat = '(' + '|'.join(['>=', '==', '>']) + ')'
            parts = re.split(pat, line, maxsplit=1)
            parts = [p.strip() for p in parts]

            info = {}
            info['package'] = parts[0]
            if len(parts) > 1:
                op, rest = parts[1:]
                if ';' in rest:
                    # Handle platform specific dependencies
                    # http://setuptools.readthedocs.io/en/latest/setuptools.html#declaring-platform-specific-dependencies
                    version, platform_deps = map(str.strip, rest.split(';'))
                    info['platform_deps'] = platform_deps
                else:
                    version = rest  # NOQA
                info['version'] = (op, version)
            yield info

    def parse_require_file(fpath):
        with open(fpath, 'r') as f:
            for line in f.readlines():
                line = line.strip()
                if line and not line.startswith('#'):
                    for info in parse_line(line):
                        yield info

    # This breaks on pip install, so check that it exists.
    packages = []
    if exists(require_fpath):
        for info in parse_require_file(require_fpath):
            package = info['package']
            if not sys.version.startswith('3.4'):
                # apparently package_deps are broken in 3.4
                platform_deps = info.get('platform_deps')
                if platform_deps is not None:
                    package += ';' + platform_deps
            packages.append(package)
    return packages


try:
    class EmptyListWithLength(list):
        def __len__(self):
            return 1

        def __repr__(self):
            return 'EmptyListWithLength()'

        def __str__(self):
            return 'EmptyListWithLength()'
except Exception:
    raise RuntimeError('FAILED TO ADD BUILD CONSTRUCTS')


NAME = 'pyflann_ibeis'


MB_PYTHON_TAG = native_mb_python_tag()  # NOQA
VERSION = version = parse_version('pyflann_ibeis/__init__.py')  # must be global for git tags

AUTHORS = ['Jon Crall', 'Marius Muja']
AUTHOR_EMAIL = 'erotemic@gmail.com'
URL = 'https://github.com/Erotemic/pyflann_ibeis'
LICENSE = 'BSD'
DESCRIPTION = 'FLANN (for IBEIS) - Fast Library for Approximate Nearest Neighbors'


KWARGS = OrderedDict(
    name=NAME,
    version=VERSION,
    author=', '.join(AUTHORS),
    author_email=AUTHOR_EMAIL,
    description=DESCRIPTION,
    long_description=parse_long_description('README.rst'),
    long_description_content_type='text/x-rst',
    url=URL,
    license=LICENSE,
    install_requires=parse_requirements('requirements/runtime.txt'),
    extras_require={
        'all': parse_requirements('requirements.txt'),
        'tests': parse_requirements('requirements/tests.txt'),
        'build': parse_requirements('requirements/build.txt'),
        'runtime': parse_requirements('requirements/runtime.txt'),
    },

    # --- PACKAGES ---
    # The combination of packages and package_dir is how scikit-build will
    # know that the cmake installed files belong in the pyflann module and
    # not the data directory.

    packages=[
        'pyflann_ibeis',
        # These are generated modules that will be created via build
        'pyflann_ibeis.lib',
    ],
    package_dir={
        'pyflann_ibeis': 'pyflann_ibeis',
        # Note: this requires that FLANN_LIB_INSTALL_DIR is set to pyflann/lib
        # in the src/cpp/CMakeLists.txt
        'pyflann_ibeis.lib': 'pyflann_ibeis/lib',
    },
    package_data={
        'pyflann_ibeis': ['py.typed', '*.pyi'],
    },

    include_package_data=False,
    # List of classifiers available at:
    # https://pypi.python.org/pypi?%3Aaction=list_classifiers
    python_requires='>=3.7',
    classifiers=[
        'Development Status :: 6 - Mature',
        'License :: OSI Approved :: BSD License',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Unix',
        'Topic :: Software Development :: Libraries :: Python Modules',
        # 'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
        'Topic :: Scientific/Engineering :: Image Recognition'
    ],
    cmake_args=[
        '-DBUILD_C_BINDINGS=ON',
        '-DBUILD_MATLAB_BINDINGS=OFF',
        '-DBUILD_EXAMPLES=OFF',
        '-DBUILD_TESTS=OFF',
        '-DBUILD_DOC=OFF',
    ],
    ext_modules=EmptyListWithLength(),  # hack for including ctypes bins
)

if __name__ == '__main__':
    """
    python -c "import pyflann_ibeis; print(pyflann_ibeis.__file__)"
    """
    import skbuild
    skbuild.setup(**KWARGS)



#Copyright 2008-2009  Marius Muja (mariusm@cs.ubc.ca). All rights reserved.
#Copyright 2008-2009  David G. Lowe (lowe@cs.ubc.ca). All rights reserved.
#
#THE BSD LICENSE
#
#Redistribution and use in source and binary forms, with or without
#modification, are permitted provided that the following conditions
#are met:
#
#1. Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#2. Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
#IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
#OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
#NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
#THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import sys
#import os
#sys.path.insert(0, os.path.split(__file__)[0]) # make python3 happy

"""
mkinit pyflann_ibeis
"""

__version__ = '2.3.0'
__author__ = 'Jon Crall'
__author_email__ = 'erotemic@gmail.com'
__url__ = 'https://github.com/Erotemic/pyflan_ibeis'


from pyflann_ibeis import exceptions
from pyflann_ibeis import flann_ctypes
from pyflann_ibeis import index

from pyflann_ibeis.exceptions import (FLANNException,)
from pyflann_ibeis.flann_ctypes import (CustomStructure, FLANNParameters,
                                        FLANN_INDEX, FlannLib, STRING,
                                        allowed_types, default_flags,
                                        define_functions, ensure_2d_array,
                                        flann, flannlib, load_flann_library,
                                        type_mappings,)
from pyflann_ibeis.index import (FLANN, index_type, set_distance_type,
                                 to_bytes,)

__all__ = ['CustomStructure', 'FLANN', 'FLANNException', 'FLANNParameters',
           'FLANN_INDEX', 'FlannLib', 'STRING', 'allowed_types',
           'default_flags', 'define_functions', 'ensure_2d_array',
           'exceptions', 'flann', 'flann_ctypes', 'flannlib', 'index',
           'index_type', 'load_flann_library', 'set_distance_type', 'to_bytes',
           'type_mappings']



from numpy import ndarray
from ctypes import Structure, c_char_p, c_void_p
from typing import Any, Callable, Dict

STRING = c_char_p


class CustomStructure(Structure):

    def __init__(self) -> None:
        ...

    def update(self, dict) -> None:
        ...

    def __getitem__(self, k):
        ...

    def __setitem__(self, k, v) -> None:
        ...

    def keys(self):
        ...


class FLANNParameters(CustomStructure):
    ...


default_flags: Any
allowed_types: Any
FLANN_INDEX = c_void_p


def load_flann_library():
    ...


flannlib: Any
libpath: Any


class FlannLib:
    build_index: Dict[Any, Callable]
    save_index: Dict[Any, Callable]
    load_index: Dict[Any, Callable]
    used_memory: Dict[Any, Callable]
    add_points: Dict[Any, Callable]
    remove_point: Dict[Any, Callable]
    find_nearest_neighbors: Dict[Any, Callable]
    find_nearest_neighbors_index: Dict[Any, Callable]
    radius_search: Dict[Any, Callable]
    compute_cluster_centers: Dict[Any, Callable]


flann: Any
type_mappings: Any


def define_functions(text: str):
    ...


def ensure_2d_array(arr: ndarray, flags: Any, **kwargs):
    ...



#!/usr/bin/env python
import numpy as np
from pyflann_ibeis import FLANN
import unittest


class Test_PyFLANN_nn(unittest.TestCase):

    def setUp(self):
        self.nn = FLANN(log_level="warning")

    ##########################################################################
    # The typical

    def test_nn_2d_2pt(self):
        self.__nd_random_test_autotune(2, 2)

    def test_nn_autotune_2d_10pt(self):
        self.__nd_random_test_autotune(2, 10)

    # def test_nn_autotune_100d_1000pt(self):
    #     self.__nd_random_test_autotune(100, 1000)

    # def test_nn_autotune_500d_100pt(self):
    #     self.__nd_random_test_autotune(500, 100)

    #
    #    ####################################################################
    #    # Stress it should handle
    #
    def test_nn_stress_1d_1pt_kmeans_autotune(self):
        self.__nd_random_test_autotune(1, 1)

    def __ensure_list(self, arg):
        if not isinstance(arg, list):
            return [arg]
        else:
            return arg

    def __nd_random_test_autotune(self, dim, N, num_neighbors=1, **kwargs):
        """
        Make a set of random points, then pass the same ones to the
        query points.  Each point should be closest to itself.
        """
        np.random.seed(0)
        x = np.random.rand(N, dim)
        xq = np.random.rand(N, dim)
        # perm = np.random.permutation(N)

        # compute ground truth nearest neighbors
        gt_idx, gt_dist = self.nn.nn(x, xq,
                                     algorithm='linear',
                                     num_neighbors=num_neighbors)

        for tp in [0.70, 0.80, 0.90]:
            nidx, ndist = self.nn.nn(x, xq,
                                     algorithm='autotuned',
                                     sample_fraction=1.0,
                                     num_neighbors=num_neighbors,
                                     target_precision=tp, checks=-2, **kwargs)

            correctness = 0.0
            for i in range(N):
                l1 = self.__ensure_list(nidx[i])
                l2 = self.__ensure_list(gt_idx[i])
                correctness += float(len(set(l1).intersection(l2))
                                     ) / num_neighbors
            correctness /= N
            self.assertTrue(correctness >= tp * 0.9,
                            'failed #1: targ_prec=%f, N=%d,correctness=%f' % (tp, N, correctness))


if __name__ == '__main__':
    """
    pytest ~/code/flann/tests/test_nn_autotune.py
    """
    unittest.main()



#!/usr/bin/env python
"""
Create a base docker image for building pyflann_ibeis
"""
from __future__ import absolute_import, division, print_function
import os
from os.path import join
import ubelt as ub


def main():

    # TODO: find a better place for root
    ROOT = join(os.getcwd())
    # ROOT = '.'
    os.chdir(ROOT)

    NAME = 'pyflann_ibeis'
    VERSION = '2.2.0'
    DOCKER_TAG = '{}-{}'.format(NAME, VERSION )

    QUAY_REPO = 'quay.io/erotemic/manylinux-for'
    DOCKER_URI = '{QUAY_REPO}:{DOCKER_TAG}'.format(**locals())

    dockerfile_fpath = join(ROOT, 'Dockerfile')

    # This docker code is very specific for building linux binaries.
    # We will need to do a bit of refactoring to handle OSX and windows.
    # But the goal is to get at least one OS working end-to-end.
    """
    Notes:
        docker run --rm -it quay.io/pypa/manylinux2014_x86_64 /bin/bash
        ---
        ls /opt/python
    """

    # BASE_IMAGE = 'quay.io/pypa/manylinux2010_x86_64'
    BASE_IMAGE = 'quay.io/pypa/manylinux2014_x86_64'

    docker_code = ub.codeblock(
        f'''
        FROM {BASE_IMAGE}

        RUN yum install lz4-devel -y
        ''')

    tags = [
        # 'cp27-cp27m'
        # 'cp35-cp35m'
        # 'cp36-cp36m',
        'cp37-cp37m',
        'cp38-cp38',
        'cp39-cp39',
        'cp310-cp310',
    ]

    pyinstall_cmds = []
    for tag in tags:
        pyinstall_cmds.append(ub.codeblock(
            fr'''
            RUN MB_PYTHON_TAG={tag} && \
            /opt/python/{tag}/bin/python -m pip install setuptools pip virtualenv -U && \
            /opt/python/{tag}/bin/python -m virtualenv ./venv-{tag} && \
            source ./venv-{tag}/bin/activate && \
            pip install pip -U && \
            pip install scikit-build cmake ninja
            '''))
    docker_code += '\n' + '\n\n'.join([*pyinstall_cmds])

    # RUN MB_PYTHON_TAG=cp27-cp27m  && \
    #     /opt/python/$MB_PYTHON_TAG/bin/python -m pip install setuptools pip virtualenv -U && \
    #     /opt/python/$MB_PYTHON_TAG/bin/python -m virtualenv ./venv-$MB_PYTHON_TAG && \
    #     source ./venv-$MB_PYTHON_TAG/bin/activate && \
    #     pip install scikit-build cmake ninja

    # RUN MB_PYTHON_TAG=cp27-cp27mu  && \
    #     /opt/python/$MB_PYTHON_TAG/bin/python -m pip install setuptools pip virtualenv -U && \
    #     /opt/python/$MB_PYTHON_TAG/bin/python -m virtualenv ./venv-$MB_PYTHON_TAG && \
    #     source ./venv-$MB_PYTHON_TAG/bin/activate && \
    #     pip install scikit-build cmake ninja

    # RUN MB_PYTHON_TAG=cp35-cp35m  && \
    #     /opt/python/$MB_PYTHON_TAG/bin/python -m pip install setuptools pip virtualenv -U && \
    #     /opt/python/$MB_PYTHON_TAG/bin/python -m virtualenv ./venv-$MB_PYTHON_TAG && \
    #     source ./venv-$MB_PYTHON_TAG/bin/activate && \
    #     pip install scikit-build cmake ninja

    # RUN MB_PYTHON_TAG=cp36-cp36m  && \
    #     /opt/python/$MB_PYTHON_TAG/bin/python -m pip install setuptools pip virtualenv -U && \
    #     /opt/python/$MB_PYTHON_TAG/bin/python -m virtualenv ./venv-$MB_PYTHON_TAG && \
    #     source ./venv-$MB_PYTHON_TAG/bin/activate && \
    #     pip install scikit-build cmake ninja

    # RUN MB_PYTHON_TAG=cp37-cp37m  && \
    #     /opt/python/$MB_PYTHON_TAG/bin/python -m pip install setuptools pip virtualenv -U && \
    #     /opt/python/$MB_PYTHON_TAG/bin/python -m virtualenv ./venv-$MB_PYTHON_TAG && \
    #     source ./venv-$MB_PYTHON_TAG/bin/activate && \
    #     pip install scikit-build cmake ninja

    # RUN MB_PYTHON_TAG=cp38-cp38  && \
    #     /opt/python/$MB_PYTHON_TAG/bin/python -m pip install setuptools pip virtualenv -U && \
    #     /opt/python/$MB_PYTHON_TAG/bin/python -m virtualenv ./venv-$MB_PYTHON_TAG && \
    #     source ./venv-$MB_PYTHON_TAG/bin/activate && \
    #     pip install scikit-build cmake ninja
    # ''')

    # docker_code2 = '\n\n'.join([ub.paragraph(p) for p in docker_code.split('\n\n')])
    docker_code2 = docker_code

    try:
        print(ub.color_text('\n--- DOCKER CODE ---', 'white'))
        print(ub.highlight_code(docker_code2, 'docker'))
        print(ub.color_text('--- END DOCKER CODE ---\n', 'white'))
    except Exception:
        pass
    with open(dockerfile_fpath, 'w') as file:
        file.write(docker_code2)

    docker_build_cli = ' '.join([
        'docker', 'build',
        '--tag {}'.format(DOCKER_TAG),
        '-f {}'.format(dockerfile_fpath),
        '.'
    ])
    print('docker_build_cli = {!r}'.format(docker_build_cli))
    if ub.argflag('--dry'):
        print('DRY RUN')
        print('WOULD RUN')
        print(docker_build_cli)
    else:
        info = ub.cmd(docker_build_cli, verbose=3, shell=True)
        if info['ret'] != 0:
            print(ub.color_text('\n--- FAILURE ---', 'red'))
            print('Failed command:')
            print(info['command'])
            print(info['err'])
            print('NOTE: sometimes reruning the command manually works')
            raise Exception('Building docker failed with exit code {}'.format(info['ret']))
        else:
            print(ub.color_text('\n--- SUCCESS ---', 'green'))

    print(ub.highlight_code(ub.codeblock(
        r'''
        # Finished creating the docker image.
        # To test / export / publish you can do something like this:

        # Test that we can get a bash terminal
        docker run -it {DOCKER_TAG} /bin/bash

        # Create a tag for the docker image
        docker tag {DOCKER_TAG} {DOCKER_URI}

        # Export your docker image to a file
        docker save -o ${ROOT}/{DOCKER_TAG}.docker.tar {DOCKER_TAG}

        # Login to a docker registry (we are using quay)

        # In some cases this works, but...
        # docker login

        # You may need to specify secret credentials
        load_secrets
        echo "QUAY_USERNAME = $QUAY_USERNAME"
        docker login -u $QUAY_USERNAME -p $QUAY_PASSWORD quay.io
        unload_secrets

        # Upload the docker image to quay.io
        docker push {DOCKER_URI}
        ''').format(NAME=NAME, ROOT=ROOT, DOCKER_TAG=DOCKER_TAG,
                    DOCKER_URI=DOCKER_URI), 'bash'))

    PUBLISH = 0
    if PUBLISH:
        cmd1 = 'docker tag {DOCKER_TAG} {DOCKER_URI}'.format(**locals())
        cmd2 = 'docker push {DOCKER_URI}'.format(**locals())
        print('-- <push cmds> ---')
        print(cmd1)
        print(cmd2)
        print('-- </push cmds> ---')


if __name__ == '__main__':
    """
    CommandLine:
        python ~/code/pyflann_ibeis/dev/docker/make_base_image.py --dry
        python ~/code/pyflann_ibeis/dev/docker/make_base_image.py
    """
    main()
